const path = require('path');
const fs = require('fs');

const getConfFile = {
    toml: () => 'port = 80\n\ndebug = true\n\n'
}

// Generate conf file if specified.
function generateConfFile(input){
    if(typeof input.confPath !== 'string')
        return false;

    let confType = input.confType || 'toml';
    if(!(confType in getConfFile))
        throw new Error('Conf type not supported: ' + confType);

    // Create config file.
    fs.mkdirSync(path.dirname(input.confPath), { recursive: true });
    fs.writeFileSync(input.confPath, getConfFile[confType]());
    return confType;
}

// Create LIB folder and main server file.
function generateMainFile(projDir){
    let src = `
const { AppServer } = require('restify-app');\n
module.exports = function init(conf){\n
    let app = new AppServer(conf);\n
    let shared = {};
    app.expose(shared);\n
    app.route(({ post, get, del, head, patch, put }) => {\n\n\t});\n
    app.on('error', function(req, res, err, send){\n\n\t});\n
    app.beforeStart = async function(){\n\n\t};\n
    app.afterStop = async function(){\n\n\t};\n
    return app;
}\n`;

    fs.mkdirSync(projDir + '/lib');
    fs.writeFileSync(projDir + '/lib/main.js', src);
}

// Create BIN folder and the run binary.
function generateRunFile({ confPath, confType }, projDir, projName){
    let cps = confPath ? `,\n\tconfPath: '${confPath}'` : '';
    let cts = confType ? `,\n\tconfType: '${confType}'` : '';
    let src = `#!node\n
const { run } = require('restify-app');
run({\n\tinit: require('../lib/main')${cps}${cts}\n});\n`;

    fs.mkdirSync(projDir + '/bin/');
    fs.writeFileSync(projDir + '/bin/' + projName + '.js', src);
}

module.exports = function init(input){
    input = input || {};

    let projDir = path.resolve(process.cwd(), input.path || '.');
    let pkgJSONPath = path.resolve(projDir, 'package.json');

    // Check for package.json
    if(!fs.existsSync(pkgJSONPath))
        throw new Error('package.json not found in: ' + pkgJSONPath);

    let pkgInfo = require(pkgJSONPath);
    let projName = input.name || pkgInfo.name || 'my-app';

    console.log('Generating basic file structure...');

    input.confType = generateConfFile(input);
    generateRunFile(input, projDir, projName);
    generateMainFile(projDir);

    // Add binary to package.json.
    pkgInfo.bin = { [projName]: 'bin/' + projName + '.js' };
    fs.writeFileSync(pkgJSONPath, JSON.stringify(pkgInfo));

    if(!('restify-app' in (pkgInfo.dependencies || [])))
        console.log('Install restify-app localy with:\n\t\tnpm i restify-app');
    console.log('Install your app run binary with:\n\t\tnpm link');
};
